//
//  ViewController.m
//  particulate
//
//  Created by Logan Cautrell on 2/11/12.
//  Copyright (c) 2012 Dreamfreeze Software. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>

#import "RootViewController.h"
#import "SettingsViewController.h"

@interface RootViewController() <UIGestureRecognizerDelegate>

@property (nonatomic, strong) SettingsViewController *settingsViewController;
@property (nonatomic, strong) CALayer *containerLayer;
@property (nonatomic, strong) CAEmitterLayer *emitterLayer;
@property (nonatomic, strong) UITapGestureRecognizer *tapper;
@property (nonatomic, strong) UIPanGestureRecognizer *panner;
@property (nonatomic, strong) CAEmitterCell *cell;

@end


@implementation RootViewController

@synthesize settingsViewController, containerLayer, emitterLayer, tapper, cell, panner;


#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];

    self.view.backgroundColor = [UIColor blackColor];
    self.tapper = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(tapped:)];
    self.tapper.delegate = self;
    [self.view addGestureRecognizer:self.tapper];
    self.panner = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(panned:)];
    [self.view addGestureRecognizer:self.panner];
    self.panner.delegate = self;
    
    // This is the tail.
    // Setup a container for the emitter.
    self.containerLayer = [CALayer layer];
    self.containerLayer.frame = CGRectMake(0, 0, self.view.bounds.size.width - self.settingsViewController.view.bounds.size.width, self.view.bounds.size.height);
    self.containerLayer.masksToBounds = YES;
    self.containerLayer.backgroundColor = [UIColor clearColor].CGColor;
    [self.view.layer addSublayer:self.containerLayer];
    
    // Setup an emitter.
    self.emitterLayer = [CAEmitterLayer layer];
    emitterLayer.frame = self.containerLayer.bounds;
    emitterLayer.emitterPosition = CGPointMake(400, 165);
	emitterLayer.renderMode = kCAEmitterLayerAdditive;
    emitterLayer.emitterShape = kCAEmitterLayerSphere;
    self.emitterLayer.backgroundColor = [UIColor clearColor].CGColor;
    
    [self.containerLayer addSublayer:self.emitterLayer];
    self.emitterLayer.emitterPosition = CGPointMake(CGRectGetMidX(self.emitterLayer.superlayer.frame), CGRectGetMidY(self.emitterLayer.superlayer.frame));

    
    // Add the emitter cell.    
    UIImage *image = [UIImage imageNamed:@"particle0.png"];
    
    self.cell = [CAEmitterCell emitterCell];
    cell.contents = (id)[image CGImage];
	cell.lifetime = 2.0f;
	cell.birthRate = 15;
	cell.velocity = 100;
	cell.emissionRange = M_PI*2;    
    emitterLayer.emitterCells = [NSArray arrayWithObjects:cell, nil];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    self.settingsViewController = [[SettingsViewController alloc] init];
    self.settingsViewController.view.frame = CGRectMake(self.view.bounds.size.width - 256, 0, 256, self.view.bounds.size.height);
    [self.view addSubview:self.settingsViewController.view];
    
    self.settingsViewController.emitterCell = self.cell;
    self.settingsViewController.emitterLayer = self.emitterLayer;
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return UIInterfaceOrientationIsLandscape(interfaceOrientation);
}


#pragma mark Implementation Methods

- (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldReceiveTouch:(UITouch *)touch {
    CGPoint point = [touch locationInView:self.view];
    CGPoint settingsViewPoint = [self.settingsViewController.view convertPoint:point fromView:self.view];
    return ![self.settingsViewController.view pointInside:settingsViewPoint withEvent:nil];
    
}

- (IBAction)tapped:(id)sender 
{
    CGPoint point = [self.tapper locationInView:self.view];
    CGPoint settingsViewPoint = [self.settingsViewController.view convertPoint:point fromView:self.view];
    if (![self.settingsViewController.view pointInside:settingsViewPoint withEvent:nil]) {
        emitterLayer.emitterPosition = point;
    }
}

- (IBAction)panned:(id)sender {
    CGPoint point = [self.panner locationInView:self.view];
    CGPoint settingsViewPoint = [self.settingsViewController.view convertPoint:point fromView:self.view];
    if (![self.settingsViewController.view pointInside:settingsViewPoint withEvent:nil]) {
        emitterLayer.emitterPosition = point;
    }
}

@end

























