//
//  SettingsViewController.m
//  particulate
//
//  Created by Logan Cautrell on 2/11/12.
//  Copyright (c) 2012 Dreamfreeze Software. All rights reserved.
//

#import "SettingsViewController.h"

#import "InfColorPickerController.h"

@interface SettingsViewController() <InfColorPickerControllerDelegate, UIPopoverControllerDelegate>

@property (nonatomic, strong) UIView *contentView;
@property (nonatomic, strong) UIPopoverController *popover;
@property (nonatomic, strong) InfColorPickerController *colorPicker;
@end


@implementation SettingsViewController

@synthesize contentView = _contentView;
@synthesize popover = _popover;
@synthesize colorPicker = _colorPicker;

@synthesize emitterCell = _emitterCell;
@synthesize emitterLayer = _emitterLayer;
@synthesize pageControl = _pageControl;
@synthesize scrollView = _scrollView;
@synthesize page1View = _page1View;
@synthesize page2View = _page2View;
@synthesize page3View = _page3View;
@synthesize velocitySlider = _velocitySlider;
@synthesize colorView = _colorView;

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    // Setup the scroll view and add our page views.
    self.contentView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, self.scrollView.bounds.size.width*3, self.scrollView.bounds.size.height)];
    [self.scrollView addSubview:self.contentView];
    self.scrollView.contentSize = self.contentView.frame.size;
    
    [self.scrollView addSubview:self.page1View];
    [self.scrollView addSubview:self.page2View];
    [self.scrollView addSubview:self.page3View];
    
    self.page1View.frame = CGRectMake(0, 0, self.page1View.frame.size.width, self.page1View.frame.size.height);
    self.page2View.frame = CGRectMake(self.page1View.frame.size.width, 0, self.page1View.frame.size.width, self.page1View.frame.size.height);
    self.page3View.frame = CGRectMake(self.page1View.frame.size.width*2, 0, self.page1View.frame.size.width, self.page1View.frame.size.height);
}

- (void)viewDidUnload
{
    self.contentView = nil;
    self.popover = nil;
    self.colorPicker = nil;
    
    self.emitterCell = nil;
    self.emitterLayer = nil;
    self.pageControl = nil;
    self.scrollView = nil;
    self.page1View = nil;
    self.page2View = nil;
    self.page3View = nil;
    self.velocitySlider = nil;
    
    [self setColorView:nil];
    [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return UIInterfaceOrientationIsLandscape(interfaceOrientation);
}


#pragma mark Action Methods

- (IBAction)pageControlValueChanged:(id)sender
{
    [self.scrollView scrollRectToVisible:CGRectMake(self.pageControl.currentPage*self.scrollView.bounds.size.width, 0, self.scrollView.bounds.size.width, self.scrollView.bounds.size.height) animated:YES];
}

- (IBAction)imageSizeChanged:(UISegmentedControl *)sender {
    NSString *imageName = [NSString stringWithFormat:@"particle%d.png", sender.selectedSegmentIndex];
    self.emitterCell.contents =  (id)[UIImage imageNamed:imageName].CGImage;
    
    [self resetEmitterCell];
}

- (IBAction)lifeTimeChanged:(UISlider *)sender {
    self.emitterCell.lifetime = sender.value;
    [self resetEmitterCell];
}

- (IBAction)lifeTimeRangeChanged:(UISlider *)sender {
    self.emitterCell.lifetimeRange = sender.value;
    [self resetEmitterCell];
}

- (IBAction)velocityChanged:(UISlider *)sender {
    self.emitterCell.velocity = sender.value;    
    [self resetEmitterCell];
}

- (IBAction)velocityRangeChanged:(UISlider *)sender {
    self.emitterCell.velocityRange = sender.value;
    [self resetEmitterCell];
}

- (IBAction)scaleChanged:(UISlider *)sender {
    
    self.emitterCell.scale = sender.value;            
    [self resetEmitterCell];
}

- (IBAction)scaleRangeChanged:(UISlider *)sender {
    self.emitterCell.scaleRange = sender.value;
    [self resetEmitterCell];
}

- (IBAction)scaleSpeedChanged:(UISlider *)sender {
    self.emitterCell.scaleSpeed = sender.value;
    [self resetEmitterCell];
}

- (IBAction)redRangeChanged:(UISlider *)sender {
    self.emitterCell.redRange = sender.value;
    [self resetEmitterCell];
}

- (IBAction)redSpeedChanged:(UISlider *)sender {
    self.emitterCell.redSpeed = sender.value;
    [self resetEmitterCell];
}

- (IBAction)blueRangeChanged:(UISlider *)sender {
    self.emitterCell.blueRange = sender.value;
    [self resetEmitterCell];
}

- (IBAction)blueSpeedChanged:(UISlider *)sender {
    self.emitterCell.blueSpeed = sender.value;
    [self resetEmitterCell];
}

- (IBAction)greenRangeChanged:(UISlider *)sender {
    self.emitterCell.greenRange = sender.value;
    [self resetEmitterCell];
}

- (IBAction)greenSpeedChanged:(UISlider *)sender {
    self.emitterCell.greenSpeed = sender.value;
    [self resetEmitterCell];
}

- (IBAction)colorButtonTapped:(UIButton *)sender {
    if (self.popover != nil)
    {
        [self.popover dismissPopoverAnimated:YES];
        self.popover = nil;
    }
    else
    {
        self.colorPicker = [InfColorPickerController colorPickerViewController];
        self.colorPicker.delegate = self;
        self.colorPicker.sourceColor = self.colorView.backgroundColor;
        self.popover = [[UIPopoverController alloc] initWithContentViewController:self.colorPicker];
        self.popover.delegate = self;
        [self.popover presentPopoverFromRect:sender.frame inView:sender.superview permittedArrowDirections:UIPopoverArrowDirectionAny animated:YES];
    }
}

- (IBAction)alphaRangeChanged:(UISlider *)sender {
    self.emitterCell.alphaRange = sender.value;
    [self resetEmitterCell];
}

- (IBAction)xAccelerationChanged:(UISlider *)sender {
    self.emitterCell.xAcceleration = sender.value;
    [self resetEmitterCell];
}

- (IBAction)yAccelerationChanged:(UISlider *)sender {
    self.emitterCell.yAcceleration = sender.value;
    [self resetEmitterCell];
}

- (IBAction)zAccelerationChanged:(UISlider *)sender {
    self.emitterCell.zAcceleration = sender.value;
    [self resetEmitterCell];
}

- (IBAction)emissionLatitudeChanged:(UISlider *)sender {
    self.emitterCell.emissionLatitude = sender.value;
    [self resetEmitterCell];
}

- (IBAction)emissionLongitudeChanged:(UISlider *)sender {
    self.emitterCell.emissionLongitude = sender.value;
    [self resetEmitterCell];
}

- (IBAction)emissionRangeChanged:(UISlider *)sender {
    self.emitterCell.emissionRange = sender.value;
    [self resetEmitterCell];
}

#pragma mark ScrollView Delegate Methods

- (void)scrollViewDidScroll:(UIScrollView *)scrollView
{
    self.pageControl.currentPage = self.scrollView.contentOffset.x / self.scrollView.bounds.size.width;
}

#pragma mark Color Picker Methods

- (void)colorPickerControllerDidFinish:(InfColorPickerController *)controller
{
    NSLog(@"here");
}

- (void)colorPickerControllerDidChangeColor:(InfColorPickerController*)controller 
{
    self.colorView.backgroundColor = self.colorPicker.resultColor;
}

- (void)popoverControllerDidDismissPopover:(UIPopoverController *)popoverController
{
    self.emitterCell.color = self.colorView.backgroundColor.CGColor;
    [self resetEmitterCell];
    
    self.colorView.backgroundColor = self.colorPicker.resultColor;
    self.colorPicker = nil;
    self.popover = nil;
}

#pragma mark Implementation Methods

- (void)resetEmitterCell 
{
    CGPoint point = self.emitterLayer.emitterPosition;
    self.emitterLayer.emitterPosition = CGPointZero;
    self.emitterLayer.emitterPosition = point;
}

@end
